/*********************************************************************
 * [SliderPuzzle スクリプト] Pieces.js
 *
 * Copyright (C) 2009,2015 ASANO Sei All Rights Reserved.
 *********************************************************************/
var Pieces_axis = {
	up : [0,-1]
	,down : [0,1]
	,left : [-1,0]
	,right : [1,0]
}
var Pieces_approachGroup = {
	up : ["up","left","right"]
	,down : ["down","left","right"]
	,left : ["up","down","left"]
	,right : ["up","down","right"]
}
/*********************************************************************
 * コンストラクタ
 * -------------------------------------------------------------------
 * [引数] rows: 行数
 *        cols: 列数
 * [戻り] なし
 *********************************************************************/
function Pieces(rows, cols, url, border) {
	this.initialize = Pieces_initialize;
	this.getPiece = Pieces_getPiece;
	this.setPiece = Pieces_setPiece;
	this.getAdjoinBlankCell = Pieces_getAdjoinBlankCell;
	this.isBlank = Pieces_isBlank;
	this.isNotBlank = Pieces_isNotBlank;
	this.shuffle = Pieces_shuffle;
	this.shuffleSub = Pieces_shuffleSub;
	this.move = Pieces_move;
	this.isCompleted = Pieces_isCompleted;
	this.checkRange = Pieces_checkRange;
	this.replace = Pieces_replace;
	this.editClassName = Pieces_editClassName;
	this.editBackgroundImage = Pieces_editBackgroundImage;
	//初期化
	this.url = url;
	this.border = border;
	this.blankCell = null;
	this.initialize(rows, cols);
}
/*********************************************************************
 * セル情報保持用に、指定した行数と列数で二次元配列を生成する。
 * -------------------------------------------------------------------
 * [引数] rows: 行数
 *        cols: 列数
 * [戻り] なし
 *********************************************************************/
function Pieces_initialize(rows, cols) {
	this.items = new Array();
	this.items.length = rows;
	for (var i=0 ; i<rows ; i++) {
		this.items[i] = new Array();
		this.items[i].length = cols; //二度目以降は、new Array(length) ではだめだった(IE6.0)ので後からlengthを設定
	}
}
/*********************************************************************
 * 指定したインデックスにセルがあるかを返却する。
 * -------------------------------------------------------------------
 * [引数] rowIndex: 行インデックス
 *        colIndex: 列インデックス
 * [戻り] セルがあればtrue
 *********************************************************************/
function Pieces_checkRange(rowIndex, colIndex) {
	if ((rowIndex < 0) || (rowIndex >= this.items.length)) return false;
	if ((colIndex < 0) || (colIndex >= this.items[0].length)) return false;
	return true;
}
/*********************************************************************
 * 指定したインデックスのセルを返却する。
 * -------------------------------------------------------------------
 * [引数] rowIndex: 行インデックス
 *        colIndex: 列インデックス
 * [戻り] セル。なければnull
 *********************************************************************/
function Pieces_getPiece(rowIndex, colIndex) {
	if (this.checkRange(rowIndex, colIndex)) {
		return this.items[rowIndex][colIndex];
	} else {
		return null;
	}
}
/*********************************************************************
 * 指定したインデックスにセルを設定する。
 * -------------------------------------------------------------------
 * [引数] rowIndex: 行インデックス
 *        colIndex: 列インデックス
 *        newPiece: ピース
 * [戻り] なし
 *********************************************************************/
function Pieces_setPiece(rowIndex, colIndex, newPiece) {
	if (this.checkRange(rowIndex, colIndex)) {
		this.items[rowIndex][colIndex] = newPiece;
	} else {
		return;
	}
}
/*********************************************************************
 * 指定したインデックスの上下左右のどれかに空白セルがあればそれを返却する。
 * -------------------------------------------------------------------
 * [引数] rowIndex: 行インデックス
 *        colIndex: 列インデックス
 * [戻り] 空白セル。上下左右どこにも空白セルがなければnull
 *********************************************************************/
function Pieces_getAdjoinBlankCell(rowIndex, colIndex) {
	if (this.isBlank(rowIndex-1, colIndex)) return this.getPiece(rowIndex-1, colIndex); //上
	if (this.isBlank(rowIndex+1, colIndex)) return this.getPiece(rowIndex+1, colIndex); //下
	if (this.isBlank(rowIndex, colIndex-1)) return this.getPiece(rowIndex, colIndex-1); //左
	if (this.isBlank(rowIndex, colIndex+1)) return this.getPiece(rowIndex, colIndex+1); //右
	return null;
}
/*********************************************************************
 * 指定したインデックスのセルが空白かを返却する。
 * -------------------------------------------------------------------
 * [引数] rowIndex: 行インデックス
 *        colIndex: 列インデックス
 * [戻り] 空白ならtrue
 *********************************************************************/
function Pieces_isBlank(rowIndex, colIndex) {
	if (this.checkRange(rowIndex, colIndex)) {
		return (!this.items[rowIndex][colIndex].hasData);
	} else {
		return false;
	}
}
/*********************************************************************
 * 指定したインデックスのセルが空白以外かを返却する。
 * -------------------------------------------------------------------
 * [引数] rowIndex: 行インデックス
 *        colIndex: 列インデックス
 * [戻り] 空白以外ならtrue
 *********************************************************************/
function Pieces_isNotBlank(rowIndex, colIndex) {
	if (this.checkRange(rowIndex, colIndex)) {
		return (this.items[rowIndex][colIndex].hasData);
	} else {
		return false;
	}
}
/*********************************************************************
 * シャッフルする。
 * -------------------------------------------------------------------
 * [引数] なし
 * [戻り] なし
 *********************************************************************/
function Pieces_shuffle() {
	if (!this.blankCell) return;
	this.shuffleSub((this.items.length * this.items[0].length) * 2, "up");
}
function Pieces_shuffleSub(count, direction) {
	if (!--count) return;
	var group = Pieces_approachGroup[direction];
	var piece = null;
	while (piece == null) {
		var axisName = group[Math.floor(Math.random() * group.length)];
		piece = this.getPiece(this.blankCell.rowIndex + Pieces_axis[axisName][0], this.blankCell.colIndex + Pieces_axis[axisName][1]);
	}
	this.move(piece);
	this.shuffleSub(count, axisName);
}
/*********************************************************************
 * 指定したセルの位置を空白セルと置換する。
 * ※実際にはオブジェクト自体ではなく各属性を入れ替える。
 * -------------------------------------------------------------------
 * [引数] item: 隣接した上下左右のどこかのセルが空白であるセル
 * [戻り] なし
 *********************************************************************/
function Pieces_move(item) {
	this.replace(item, this.blankCell);
	this.blankCell = item;
}
/*********************************************************************
 * 指定したセル同士の位置を置換する。
 * ※実際にはオブジェクト自体ではなく各属性を入れ替える。
 * -------------------------------------------------------------------
 * [引数] item1: セル
 *        item2: セル
 * [戻り] なし
 *********************************************************************/
function Pieces_replace(item1, item2) {
	var hasData   = item2.hasData;
	var number    = item2.number;
	var innerHTML = item2.innerHTML;
	var imgPos    = item2.style.backgroundPosition;
	var onclick   = item2.onclick;

	item2.hasData   = item1.hasData;
	item2.number    = item1.number;
	item2.innerHTML = item1.innerHTML;
	item2.style.backgroundPosition = item1.style.backgroundPosition;
	item2.onclick   = item1.onclick;
	this.editClassName(item2);
	this.editBackgroundImage(item2);

	item1.hasData   = hasData;
	item1.number    = number;
	item1.innerHTML = innerHTML;
	item1.style.backgroundPosition = imgPos;
	item1.onclick   = onclick;
	this.editClassName(item1);
	this.editBackgroundImage(item1);
}
/*********************************************************************
 * 指定したセルに背景画像を設定する。
 * -------------------------------------------------------------------
 * [引数] item: セル
 * [戻り] なし
 *********************************************************************/
function Pieces_editBackgroundImage(item) {
	if ((!showLabel && item.hasData)
			|| (item.hasData && (item.validNumber == item.number))) {
		item.style.backgroundImage = "url(" + item.url + ")";
	} else {
		item.style.backgroundImage = "";
	}
}
/*********************************************************************
 * 指定したセルが空白セルか否かによって、CSSクラス名を振り分けて設定する。
 * -------------------------------------------------------------------
 * [引数] item: セル
 * [戻り] なし
 *********************************************************************/
function Pieces_editClassName(item) {
	if (item.hasData) {
		item.className = 'piece_image'
			+ (SliderPuzzle_currentData.border ? ' piece_border' : '')
			+ (item.number == item.validNumber ? ' hasValidData' : ' hasData')
		;
	} else {
		item.className = 'piece_image noData';
	}
}
/*********************************************************************
 * パズルが完成したかを返却する。
 * -------------------------------------------------------------------
 * [引数] なし
 * [戻り] パズルが完成していればtrue
 *********************************************************************/
function Pieces_isCompleted() {
	for (var i=0 ; i<this.items.length ; i++) {
		for (var j=0 ; j<this.items[i].length ; j++) {
			if (this.items[i][j].validNumber != this.items[i][j].number) return false;
		}
	}
	return true;
}
