/*********************************************************************
 * [SliderPuzzle スクリプト] SliderPuzzle.js
 *
 * Copyright (C) 2009,2015 ASANO Sei All Rights Reserved.
 *********************************************************************/
// デバッグモード
var SliderPuzzle_debugMode = false;

// バージョン番号
var SliderPuzzle_version = '2.1';

//現在選択されている画像の情報
var SliderPuzzle_currentData = null;

var SliderPuzzle_debugAreaObj = null;
var SliderPuzzle_puzzleObj = null;
var SliderPuzzle_imgPrefix = 'img';
var isfinish;
var pieces = null;
/*********************************************************************
 * 初期処理
 *********************************************************************/
function SliderPuzzle_initialize() {
	SliderPuzzle_initializeDebugMessageArea();
	SliderPuzzle_setDebugMessageLine('[function] initialize');
	SliderPuzzle_puzzleObj = document.getElementById('puzzleArea');
	document.getElementById('version').innerHTML = SliderPuzzle_version;
	document.getElementById('mainArea').style.display = '';

	//画像リストを生成し、開始indexの指定があれば選択indexに設定する
	SliderPuzzle_generateImageList(SliderPuzzle_getStartIndex());

	//選択中の画像でパズルを作成する
	SliderPuzzle_changeImageList();
}
/*-------------------------------------------------------------------*
 * リクエストパラメータから取得したパズル選択用のインデックスを返却する。
 * 取得できなければ最小インデックスが指定されたと見なす。
 *
 * ゼロより小さな値、または数値以外を取得した場合はゼロが、
 * 最終パズルのインデックスより大きな値を取得した場合は
 * 最終パズルのインデックスが指定されたと見なす。
 *
 * [引数] なし
 * [戻り] パズル選択用のインデックス
 *-------------------------------------------------------------------*/
function SliderPuzzle_getStartIndex() {
	SliderPuzzle_appendDebugMessageLine('[function] getStartIndex');
	if (userSetting.imageList.length == 0) return -1;

	var index = 0;
	if (!window.location.search) return index;
	var param = window.location.search.substr(1).split('&'); //?を除外して&で分割
	for (var i=0 ; i<param.length ; i++) {
		var item = param[i].split('='); //名前と値に分割
		if (item.length!=2) continue;
		var start = item[1];
		if (item[0]=='start'){
			if (!isNaN(start)) {
				index = start;
				if (start >= userSetting.imageList.length) {
					index = userSetting.imageList.length - 1;
				}
			}
			break;
		}
	}
	SliderPuzzle_appendDebugMessageLine('start index: ' + index);
	return index;
}
/*-------------------------------------------------------------------*
 * 画像選択プルダウンリストの生成。
 *
 * [引数] startIndex : 選択インデックス
 * [戻り] なし
 *-------------------------------------------------------------------*/
function SliderPuzzle_generateImageList(startIndex) {
	SliderPuzzle_appendDebugMessageLine('[function] generateImageList');
	common_removeAllChildren(SliderPuzzle_puzzleObj);
	if (!userSetting.imageList.length) return;
	var selectObj = document.getElementById('puzzleList');
	for (var i=0 ; i<userSetting.imageList.length ; i++) {
		var item = userSetting.imageList[i];
		item.borderWidth = (item.border ? 2 : 0); //枠線ありなら2px（左右1pxずつ）を設定
		var caption = item.caption ? item.caption : 'No.' + (i+1);
		if (!item.src) {
			caption = 'ERROR (NO IMAGE)';
			item.invalid = true;
		}
		selectObj.appendChild(common_generateOption(i, caption));
	}
	selectObj.selectedIndex = startIndex;
}
/*********************************************************************
 * 画像選択処理。現在、リストで選択されている画像を生成し、
 * パズルを作成する。ただしファイル名の指定がなければ生成しない。
 *********************************************************************/
function SliderPuzzle_changeImageList() {
	SliderPuzzle_setDebugMessageLine('[function] changeImageList');
	common_removeAllChildren(SliderPuzzle_puzzleObj);

	var currentIndex = common_getSelectedOptionIndex('puzzleList');
	SliderPuzzle_appendDebugMessageLine('currentIndex=' + currentIndex);
	if (currentIndex < 0) return;
	SliderPuzzle_currentData = userSetting.imageList[currentIndex];
	if (SliderPuzzle_currentData.invalid) return; //指定が不正
	if (!SliderPuzzle_currentData.src) { //画像ファイル名の指定なし
		item.invalid = true;
		return;
	}
	if (SliderPuzzle_currentData.calculated) { //生成済
		SliderPuzzle_generatePuzzle();
		return;
	}
	//未生成
	if (SliderPuzzle_currentData.viewWidth && SliderPuzzle_currentData.viewHeight) {
		SliderPuzzle_currentData.img = new Image(SliderPuzzle_currentData.viewWidth, SliderPuzzle_currentData.viewHeight);
	} else {
		SliderPuzzle_currentData.img = new Image();
	}
	SliderPuzzle_currentData.img.src = userSetting.imageDir + '/' + SliderPuzzle_currentData.src;
	SliderPuzzle_currentData.img.onload = SliderPuzzle_afterLoadImage;
}
/*-------------------------------------------------------------------*
 * リストで選択されている、読込済＆分割数算出済の画像でパズルを生成。
 *-------------------------------------------------------------------*/
function SliderPuzzle_generatePuzzle() {
	SliderPuzzle_appendDebugMessageLine('[function] generatePuzzle');
	pieces = new Pieces(
				  SliderPuzzle_currentData.rows
				, SliderPuzzle_currentData.cols
				, SliderPuzzle_currentData.img.src
				, SliderPuzzle_currentData.border
	);
	SliderPuzzle_createPieces();
	pieces.shuffle();
}
/*********************************************************************
 * 画像ロード完了後処理。画像分割数の取得とパズルの生成。
 *********************************************************************/
function SliderPuzzle_afterLoadImage() {
	SliderPuzzle_appendDebugMessageLine('[function] afterLoadImage');
	SliderPuzzle_appendDebugMessageLine('画像読込完了 - ' + this.src);
	var calcOk = SliderPuzzle_calcSplitCount(SliderPuzzle_currentData);
	SliderPuzzle_appendDebugMessageLine('calcOk - ' + calcOk);
	if (!calcOk) return;
	SliderPuzzle_appendDebugMessageLine('画像情報 - ' + common_getAttributeList(SliderPuzzle_currentData).join(', '));	SliderPuzzle_generatePuzzle();
}
/*-------------------------------------------------------------------*
 * [Imageオブジェクトのonloadで実行]
 * 未計算の場合のみ、画像の分割数を算出（計算済なら使い回し）
 *
 * [引数] なし
 * [戻り] 妥当な値を算出できればtrue
 *-------------------------------------------------------------------*/
function SliderPuzzle_calcSplitCount() {
	SliderPuzzle_appendDebugMessageLine('[function] calcSplitCount');
	var errMsg = '画像を分割できません。\n設定ファイル内の分割数、または画像サイズを確認してください。';

	//一度算出した結果、エラーとなった場合
	if (SliderPuzzle_currentData.invalid) {
		alert(errMsg + ' (1)');
		return false;
	}
	//各種値を算出済
	if (SliderPuzzle_currentData.calculated) return true;

	var rows = SliderPuzzle_checkRowCol(SliderPuzzle_currentData.rows);
	var cols = SliderPuzzle_checkRowCol(SliderPuzzle_currentData.cols);
	// ユーザー指定の分割数が妥当な範囲内の場合、必要な各種サイズを求めて抜ける
	if (rows && cols) {
		SliderPuzzle_appendDebugMessageLine('ユーザー指定の分割数が妥当な範囲内(' + rows + ',' + cols + ')');
		SliderPuzzle_calcPieceSize(SliderPuzzle_currentData);
		SliderPuzzle_currentData.calculated = true;
		return true;
	}
	SliderPuzzle_appendDebugMessageLine('ユーザー指定が不正または未指定(' + rows + ',' + cols + ')');
	// ユーザー指定が不正または未指定の場合、分割数を算出
	var sizeInfo = new PieceSizeInfo();
	SliderPuzzle_appendDebugMessageLine('sizeInfo.calc('+SliderPuzzle_currentData.img.width+','+ SliderPuzzle_currentData.img.height+')');
	sizeInfo.calc(SliderPuzzle_currentData.img.width, SliderPuzzle_currentData.img.height);
	var rowObj = sizeInfo.getOptimalRow();
	var colObj = sizeInfo.getOptimalCol();
	if (!rowObj || !colObj) { //最適分割数を算出できず
		SliderPuzzle_currentData.invalid = true;
		alert(errMsg + ' (2)');
		return false;
	}
	SliderPuzzle_currentData.rows  = rowObj.count;
	SliderPuzzle_currentData.cols  = colObj.count;
	SliderPuzzle_currentData.width = rowObj.size;
	SliderPuzzle_currentData.height= colObj.size;
	SliderPuzzle_calcPieceSize(SliderPuzzle_currentData);
	SliderPuzzle_currentData.calculated = true;
	return true;
}
// ピースサイズおよび画像の表示サイズを算出
// item: 選択中の画像情報オブジェクト
function SliderPuzzle_calcPieceSize(item) {
	SliderPuzzle_appendDebugMessageLine('[function] calcPieceSize');

	// ピースサイズを算出
	item.width = parseInt(item.img.width / item.cols);
	item.height= parseInt(item.img.height/ item.rows);

	//画像の表示サイズ（ピースサイズ×縦横の分割数）を算出
	item.totalWidth  = item.rows * item.width;
	item.totalHeight = item.cols * item.height;
}
/*********************************************************************
 * 画像の分割数チェック
 *
 * [引数] なし
 * [戻り] なし
 *********************************************************************/
function SliderPuzzle_checkRowCol(val) {
	var result = false;
	if (val == null) return null;
	val = parseInt(val);
	if (isNaN(val)) return null;
	if (val < 3) return null;
	if (val > 10) return null;
	return val;
}
/*********************************************************************
 * 縦横の分割数を元に、ピースとなるオブジェクトを生成する。
 *********************************************************************/
function SliderPuzzle_createPieces() {
	SliderPuzzle_appendDebugMessageLine('[function] createPieces');

	isfinish = false;
	document.getElementById('completeMsg').innerHTML = '&nbsp;';
	showLabel = SliderPuzzle_currentData.showLabel;

	//パズル領域の子要素を全削除
	var area = SliderPuzzle_puzzleObj;
	common_removeAllChildren(area);

	//パズル領域全体のサイズを指定
	area.style.width  = SliderPuzzle_currentData.img.width  + "px";
	area.style.height = SliderPuzzle_currentData.img.height + "px";

	//
	var rowpos = 0;
	var colpos = 0;
	var className = 'piece_image hasValidData' + (SliderPuzzle_currentData.border ? ' piece_border' : '');
	for (var i=1 ; i<=SliderPuzzle_currentData.rows ; i++) {
		//div line 追加
		var lineDiv = common_generateElement('div', null, {'className':'piece_line'});
		area.appendChild(lineDiv);
		for (var j=1 ; j<=SliderPuzzle_currentData.cols ; j++) {
			//div image 追加
			var piece = SliderPuzzle_createPieceCell(i, j, rowpos, colpos, className, showLabel);
			lineDiv.appendChild(piece);
			pieces.items[piece.rowIndex][piece.colIndex] = piece;
			colpos -= SliderPuzzle_currentData.width;
		}
		colpos = 0;
		rowpos -= SliderPuzzle_currentData.height;
	}
	//右下セルを空白に
	pieces.blankCell = pieces.items[SliderPuzzle_currentData.rows-1][SliderPuzzle_currentData.cols-1];
	pieces.blankCell.hasData = false;
	pieces.blankCell.className = 'noData';
	pieces.blankCell.innerHTML = '&nbsp';
	pieces.blankCell.onclick = null;
	pieces.blankCell.style.backgroundImage = '';
}
// ピース生成
function SliderPuzzle_createPieceCell(row, col, rowpos, colpos, className, showLabel) {
	//SliderPuzzle_appendDebugMessageLine('[function] createPieceCell');
	var piece = common_generateElement(
		'div',
		null,
		null
	);
	piece.id = SliderPuzzle_imgPrefix + row + '_' + col;
	piece.className = className;
	piece.hasData = true;
	piece.rowIndex = row - 1;
	piece.colIndex = col - 1;
	piece.validNumber = ((piece.rowIndex * SliderPuzzle_currentData.cols) + piece.colIndex) + 1;
	piece.number = piece.validNumber;
	piece.url = pieces.url;
	if (showLabel) piece.innerHTML = SliderPuzzle_editNumberSpan(piece.number);
		else piece.innerHTML = '&nbsp;';
	piece.onclick = SliderPuzzle_clickProc;
	piece.style.backgroundImage = 'url(' + SliderPuzzle_currentData.img.src + ')';
	piece.style.backgroundSize = SliderPuzzle_currentData.img.width + 'px ' + SliderPuzzle_currentData.img.height + 'px';
	piece.style.backgroundPosition = colpos + 'px ' + rowpos + 'px';
	piece.style.width  = SliderPuzzle_currentData.width  + 'px';
	piece.style.height = SliderPuzzle_currentData.height + 'px';
	piece.style.lineHeight = piece.style.height;
	return piece;
}
// ピース番号SPAN生成
function SliderPuzzle_editNumberSpan(num) {
	return '<span class="num">' + num + '</span>';
}
/*********************************************************************
 * ピースのクリックイベントハンドラ
 *********************************************************************/
function SliderPuzzle_clickProc(event) {
	SliderPuzzle_setDebugMessageLine('[function] clickProc');

	if (isfinish) return;
	//クリックしたセルと空白セルを置換
	var adjoinCell = pieces.getAdjoinBlankCell(this.rowIndex, this.colIndex);
	if (!adjoinCell) return;
	pieces.move(this);
	//完成時の処理
	if (pieces.isCompleted()) {
		isfinish = true;
		//ラベルを全消去
		SliderPuzzle_clearLabel();
		//空白セルの画像を表示
		pieces.blankCell.innerHTML = '&nbsp;';
		pieces.blankCell.hasData = true;
		pieces.editClassName(pieces.blankCell);
		pieces.editBackgroundImage(pieces.blankCell);
		//パズル完成時の挙動で振り分け
		SliderPuzzle_appendDebugMessageLine('kind=' + userSetting.afterFinish.kind);
		switch (userSetting.afterFinish.kind) {
			case 0: //ダイアログメッセージ
				alert(userSetting.afterFinish.dialogMessage);
				break;
			case 1: //テキストメッセージ
				document.getElementById('completeMsg').innerHTML = userSetting.afterFinish.textMessage;
				break;
			case 2: //メッセージダイアログを表示後、指定したページにジャンプ
				alert(userSetting.afterFinish.dialogMessage);
				location.href = userSetting.afterFinish.jumpURL;
				break;
			case 3: //メッセージダイアログを表示後、次のパズルを表示（ランダム）
				alert(userSetting.afterFinish.dialogMessage);
				SliderPuzzle_gotoNextPuzzle(true);
				break;
			case 4: //メッセージダイアログを表示後、次のパズルを表示（imageListで指定した順番通り）
				alert(userSetting.afterFinish.dialogMessage);
				SliderPuzzle_gotoNextPuzzle(false);
				break;
			default:
				alert(userSetting.afterFinish.dialogMessage);
		}
	}
}
// 次のパズル
function SliderPuzzle_gotoNextPuzzle(isRandom) {
	if (userSetting.imageList.length < 1) return;
	var puzzleList = document.getElementById('puzzleList');
	var selectedIndex = puzzleList.selectedIndex;
	if (isRandom) { //ランダムの場合
		var list = [];
		for (var i=0 ; i<userSetting.imageList.length ; i++) {
			if (selectedIndex == i) continue;
			list.push(i);
		}
		if (list.length == 1) {
			puzzleList.selectedIndex = list[0];
		} else {
			puzzleList.selectedIndex = list[common_random(0, list.length)];
		}
	} else { //ランダム以外の場合は画像リストの次
		if (++selectedIndex > (userSetting.imageList.length-1)) {
			puzzleList.selectedIndex = 0;
		} else {
			puzzleList.selectedIndex = selectedIndex;
		}
	}
	SliderPuzzle_changeImageList();
}
// ピース番号クリア
function SliderPuzzle_clearLabel() {
	for (var i=0 ; i<pieces.items.length ; i++) {
		var row = pieces.items[i];
		for (var j=0 ; j<pieces.items[i].length ; j++) {
			pieces.items[i][j].innerHTML = '&nbsp;';
		}
	}
}
/*********************************************************************
 * デバッグエリア操作
 *********************************************************************/
// 初期化（これを呼ぶ前にデバッグエリアにアクセスしないこと）
function SliderPuzzle_initializeDebugMessageArea() {
	SliderPuzzle_debugAreaObj = document.getElementById('debugArea');
	SliderPuzzle_clearDebugMessage();
	SliderPuzzle_debugAreaObj.style.display = (SliderPuzzle_debugMode ? '' : 'none');
}
// クリア
function SliderPuzzle_clearDebugMessage() {
	if (SliderPuzzle_debugMode) SliderPuzzle_debugAreaObj.value = new Date() + '\n';
}
// 文字列を追加
function SliderPuzzle_appendDebugMessage(str) {
	if (SliderPuzzle_debugMode) SliderPuzzle_debugAreaObj.value += str;
}
// 行を追加
function SliderPuzzle_appendDebugMessageLine(line) {
	SliderPuzzle_appendDebugMessage(line + '\n');
}
// 文字列を設定
function SliderPuzzle_setDebugMessage(str) {
	if (SliderPuzzle_debugMode) SliderPuzzle_debugAreaObj.value = str;
}
// 行を設定
function SliderPuzzle_setDebugMessageLine(line) {
	SliderPuzzle_setDebugMessage(line + '\n');
}
common_addEvent(window, 'load', SliderPuzzle_initialize);
