/*********************************************************************
 * DOM要素にイベントハンドラを追加する
 *--------------------------------------------------------------------
 * [引数] element : 要素オブジェクト
 *        eventName: イベント名。'load'、'change'、など
 *        func : イベントハンドラとして登録する関数オブジェクト
 * [戻り] イベントハンドラを追加できたらtrue
 *********************************************************************/
function common_addEvent(element, eventName, func){
	var result = false;
	if (element.addEventListener)	{ //Firefox
		element.addEventListener(eventName, func, false);
		result = true;
	} else if (element.attachEvent) { //IE
		element.attachEvent('on' + eventName, func);
		result = true;
	}
	return result;
}
/*********************************************************************
 * 指定された要素の子要素を全部削除する。
 *--------------------------------------------------------------------
 * [引数] parent: 要素
 * [戻り] なし
 *********************************************************************/
function common_removeAllChildren(parent) {
	if (!parent) return;
	while (parent.firstChild) parent.removeChild(parent.firstChild);
}
/*********************************************************************
 * 指定された要素の子要素を全部削除する。
 *--------------------------------------------------------------------
 * [引数] id: 要素のid
 * [戻り] 指定されたidの要素
 *********************************************************************/
function common_removeAllChildrenById(id) {
	var parent = document.getElementById(id);
	common_removeAllChildren(parent);
	return parent;
}
/*********************************************************************
 * 要素を生成して返す。
 *--------------------------------------------------------------------
 * [引数] tagName : 生成する要素のHTMLタグ
 *        text : 子要素に追加するテキスト
 *        param : 要素の設定。属性名と値のセット
 * [戻り] 生成したオブジェクト
 *********************************************************************/
function common_generateElement(tagName, text, param) {
	var element = document.createElement(tagName);
	if (text) element.appendChild(document.createTextNode(text));
	if (param) {
		for (var name in param) {
			element[name] = param[name];
		}
	}
	return element;
}
/*-------------------------------------------------------------------*
 * OPTION要素を生成して返す。
 *--------------------------------------------------------------------
 * [引数] id : 対応するオブジェクトのid
 *        text : ラベルとなる文字列
 * [戻り] 生成したオブジェクト
 *-------------------------------------------------------------------*/
function common_generateOption(val, text) {
	return common_generateElement('OPTION', text, {'value':val});
}
/*********************************************************************
 * 指定されたSELECT要素の選択中のindexを返す。
 *--------------------------------------------------------------------
 * [引数] id: 要素のid
 * [戻り] index
 *********************************************************************/
function common_getSelectedOptionIndex(id) {
	var obj = document.getElementById(id);
	if (!obj) obj = document.getElementsByName(id);
	return obj.selectedIndex;
}
/*********************************************************************
 * 指定されたオブジェクトの属性一覧を返す。
 *--------------------------------------------------------------------
 * [引数] obj: オブジェクト
 * [戻り] 属性一覧の配列
 *********************************************************************/
function common_getAttributeList(obj) {
	var array = [];
	if (!obj) return array;
	for (var name in obj) {
		try {
			array.push(name + '=' + obj[name]);
		} catch(e) {
			array.push(name + '=[ERROR]');
		}
	}
	return array;
}
/*********************************************************************
 * min～(max-1)間の整数の乱数を返す。
 *--------------------------------------------------------------------
 * [引数] min : 最小値
 *        max : 最大値
 * [戻り] 生成した乱数（整数）
 *********************************************************************/
function common_random(min, max) {
	var wkmax = max - min;
	var val= Math.floor(Math.random() * wkmax);
	return (val + min);
}
